#include "PX_Game_Instance.h"


px_void PX_GameInstanceSoundPlay(px_void* userptr, px_byte* buffer, px_int size)
{
	px_int i;
	px_int count = size / 4;//2 bytes 2 channels
	PX_GameInstance* pGameInstance = (PX_GameInstance*)userptr;
	px_short* pPCM = (px_short*)buffer;
	for (i=0;i<count;i++)
	{
		px_float out=0;
		PX_PianoModelGo(&pGameInstance->pianoModel, &out, 1);
		pPCM[0] = (px_int16)(out * 32768*pGameInstance->volume);
		pPCM[1] = (px_int16)(out * 32768 * pGameInstance->volume);
		pPCM += 2;
	}
}

px_void PX_GameInstanceOnTargetOn(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_GameInstance* pGameInstance = (PX_GameInstance*)ptr;
	PX_Object_ScorePanelAddScore(pGameInstance->scorepanel, PX_Object_Event_GetInt(e));
}

px_void PX_GameInstanceOnTargetMiss(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_GameInstance* pGameInstance = (PX_GameInstance*)ptr;
	PX_Object_ScorePanelDisableCombo(pGameInstance->scorepanel);
}

#include "PX_Resource_ansi24.h"
#include "PX_Resource_ansi32.h"
#include "PX_Resource_ansi64.h"

px_void PX_GameInstanceCreateGameObject(PX_GameInstance* pGameInstance)
{
	pGameInstance->piano_keyboard = PX_Object_PianoCreate(&pGameInstance->pruntime->mp_game, 0, 0, 0, 0, 0);
	PX_WorldAddObject(&pGameInstance->world, pGameInstance->piano_keyboard);

	pGameInstance->clickbar = PX_Object_ClickBarCreate(pGameInstance->pruntime, &pGameInstance->world, &pGameInstance->fm24);
	PX_WorldAddObject(&pGameInstance->world, pGameInstance->clickbar);

	pGameInstance->scorepanel = PX_Object_ScorePanelCreate(pGameInstance->pruntime, &pGameInstance->world, &pGameInstance->fm64, &pGameInstance->fm32,pGameInstance->name,(px_int)pGameInstance->speed);
	PX_WorldAddObject(&pGameInstance->world, pGameInstance->scorepanel);

	PX_ObjectRegisterEvent(pGameInstance->clickbar, PX_OBJECT_CLICKBAR_EVENT_MISS, PX_GameInstanceOnTargetMiss, pGameInstance);
	PX_ObjectRegisterEvent(pGameInstance->clickbar, PX_OBJECT_CLICKBAR_EVENT_ADDSCORE, PX_GameInstanceOnTargetOn, pGameInstance);


	PX_SoundPlaySetUserRead(&pGameInstance->pruntime->soundplay, PX_GameInstanceSoundPlay, pGameInstance);
}

px_void PX_GameInstanceOnCounterDownTrigger(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_GameInstance* pGameInstance = (PX_GameInstance*)ptr;
	PX_WorldRemoveObject(&pGameInstance->world, pObject);
	PX_GameInstanceCreateGameObject(pGameInstance);
	pGameInstance->state = PX_GAMEINSTANCE_STATE_GAMEING;
}



px_void PX_GameInstanceOnLoadingDone(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_GameInstance* pGameInstance = (PX_GameInstance*)ptr;
	pGameInstance->state = PX_GAMEINSTANCE_STATE_COUNTERDOWN;
	pGameInstance->CounterDown = PX_Object_CounterDownCreate(&pGameInstance->pruntime->mp_game,0,pGameInstance->pruntime->surface_width/2.f, pGameInstance->pruntime->surface_height/2.f,64,&pGameInstance->fm64);
	PX_Object_CounterDownSetValue(pGameInstance->CounterDown, 3000);
	PX_WorldAddObject(&pGameInstance->world, pGameInstance->CounterDown);
	PX_ObjectRegisterEvent(pGameInstance->CounterDown, PX_OBJECT_EVENT_EXECUTE, PX_GameInstanceOnCounterDownTrigger, pGameInstance);
}

px_void PX_GameInstanceCreateLoadingObject(PX_GameInstance* pGameInstance)
{
	pGameInstance->Loading = PX_Object_InitializeCreate(pGameInstance->pruntime,&pGameInstance->world, &pGameInstance->fm24,&pGameInstance->pianoModel, pGameInstance->keyparam, &pGameInstance->soundboardparam);
	PX_WorldAddObject(&pGameInstance->world, pGameInstance->Loading);
	PX_ObjectRegisterEvent(pGameInstance->Loading, PX_OBJECT_INITIALIZE_EVENT_DONE, PX_GameInstanceOnLoadingDone, pGameInstance);
	pGameInstance->state = PX_GAMEINSTANCE_STATE_LOADING;
}

px_void PX_GameInstanceOnConfigDone(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	px_int i;
	PX_GameInstance* pGameInstance = (PX_GameInstance*)ptr;
	PX_Object_GameTuner* pDesc = PX_ObjectGetDesc(PX_Object_GameTuner, pObject);
	//////////////////////////////////////////////////////////////////////////
	//load mode

	for (i=0;i<88;i++)
	{
		pGameInstance->keyparam[i] = pDesc->Piano.keys[i].param;
	}
	pGameInstance->soundboardparam = pDesc->Piano.soundboard.param;
	pGameInstance->volume = pDesc->volume;
	pGameInstance->speed = pDesc->speed;

	PX_MidiSetQuarterNoteDuration(&pGameInstance->Midi, 1440-(px_int)(pGameInstance->speed/1000.f*1200));

	PX_WorldRemoveObject(&pGameInstance->world, pObject);
	PX_GameInstanceCreateLoadingObject(pGameInstance);
	pGameInstance->state = PX_GAMEINSTANCE_STATE_LOADING;
}

px_void PX_GameInstanceCreateConfigObject(PX_GameInstance* pGameInstance)
{
	pGameInstance->Config = PX_Object_GameTunerCreate(pGameInstance->pruntime, &pGameInstance->world, pGameInstance->keyparam,&pGameInstance->soundboardparam,(px_int)pGameInstance->speed);
	PX_WorldAddObject(&pGameInstance->world, pGameInstance->Config);
	PX_ObjectRegisterEvent(pGameInstance->Config, PX_OBJECT_GAMETUNE_EVENT_DONE, PX_GameInstanceOnConfigDone, pGameInstance);
}


px_bool PX_GameInstanceInitialize(PX_Runtime* pruntime, PX_GameInstance* pGameInstance, const px_char name[], px_float speed, px_byte midi_data[], px_int midiSize)
{
#include "core/PX_Piano_mod.h"
	PX_memset(pGameInstance, 0, sizeof(PX_GameInstance));
	pGameInstance->pruntime = pruntime;
	pGameInstance->last_leftHandle = 33;
	pGameInstance->last_rightHandle = 45;
	pGameInstance->speed = speed;
	pGameInstance->volume = 1;
	pGameInstance->state = PX_GAMEINSTANCE_STATE_CONFIG;
	PX_memcpy(pGameInstance->keyparam, piano_mod_default, sizeof(pGameInstance->keyparam));
	PX_memcpy(&pGameInstance->soundboardparam, piano_mod_default + sizeof(pGameInstance->keyparam),sizeof(pGameInstance->soundboardparam) );

	PX_strcpy(pGameInstance->name, name, sizeof(pGameInstance->name));

	if(!PX_FontModuleInitialize(&pruntime->mp_game, &pGameInstance->fm24))return PX_FALSE;
	if (!PX_FontModuleInitialize(&pruntime->mp_game, &pGameInstance->fm32))return PX_FALSE;
	if (!PX_FontModuleInitialize(&pruntime->mp_game, &pGameInstance->fm64))return PX_FALSE;
	if (!PX_FontModuleLoad(&pGameInstance->fm24, ansi24,sizeof(ansi24)))return PX_FALSE;
	if (!PX_FontModuleLoad(&pGameInstance->fm32, ansi32, sizeof(ansi32)))return PX_FALSE;
	if (!PX_FontModuleLoad(&pGameInstance->fm64, ansi64, sizeof(ansi64)))return PX_FALSE;

	if (!PX_MidiInitialize(&pGameInstance->Midi, &pruntime->mp_game,PX_NULL,pGameInstance))
	{
		return PX_FALSE;
	}

	if (!PX_MidiLoad(&pGameInstance->Midi,midi_data,midiSize))
	{
		return PX_FALSE;
	}
	

	if (PX_MidiGetTrackCount(&pGameInstance->Midi)>PX_GAMEINSTANCE_MAX_SUPPORT_TRACKS)
	{
		return PX_FALSE;
	}


	if (!PX_WorldInitialize(&pruntime->mp_game, &pGameInstance->world, pruntime->surface_width, pruntime->surface_height, pruntime->surface_width, pruntime->surface_height, 8 * 1024))
		return PX_FALSE;

	PX_WorldSetAuxiliaryLineColor(&pGameInstance->world, PX_COLOR(0, 0, 0, 0));

	PX_GameInstanceCreateConfigObject(pGameInstance);
	
	return PX_TRUE;

}




px_void PX_GameInstanceUpdateGaming(PX_GameInstance* pGameInstance, px_dword elapsed)
{
	px_int i = 0;
	px_int itrack;
	pGameInstance->elapsed_reserved += (px_int)elapsed;

	while (pGameInstance->elapsed_reserved >= PX_GAMEINSTANCE_ELAPSED_ATOM)
	{
		pGameInstance->elapsed_reserved -= PX_GAMEINSTANCE_ELAPSED_ATOM;
		pGameInstance->elapsed += PX_GAMEINSTANCE_ELAPSED_ATOM;
		pGameInstance->piano_keyboard->Width = pGameInstance->pruntime->surface_width * 1.0f;
		pGameInstance->piano_keyboard->Height = 96;
		pGameInstance->piano_keyboard->x = (pGameInstance->pruntime->surface_width - pGameInstance->piano_keyboard->Width) / 2;
		pGameInstance->piano_keyboard->y = pGameInstance->pruntime->surface_height - pGameInstance->piano_keyboard->Height - 1;

		pGameInstance->world.surface_width = pGameInstance->pruntime->surface_width;
		pGameInstance->world.surface_height = pGameInstance->pruntime->surface_height;

		for (itrack = 0; itrack < PX_MidiGetTrackCount(&pGameInstance->Midi); itrack++)
		{
			pGameInstance->GameTrackElapsed[itrack] -= PX_GAMEINSTANCE_ELAPSED_ATOM;
			if (pGameInstance->GameTrackElapsed[itrack] <= 0)
			{
				px_int tick = 0;
				px_float elapsedms = 0;
				px_float step, x;
				step = pGameInstance->piano_keyboard->Width / 88.0f;
				while (PX_TRUE)
				{
					if (PX_MidiGetNextType(&pGameInstance->Midi, itrack) == PX_MIDI_NEXT_TYPE_OPCODE)
					{
						PX_Midi_Note note = PX_MidiReadNote(&pGameInstance->Midi, itrack);
						if (note.note != -1)
						{
							px_int index;
							index = PX_MidiNoteToPianoKey(note.note);
							if (index >= 0 && index < 88)
							{
								PX_Object* pNewNote;
								x = index * step + step / 2;
								pNewNote = PX_Object_GameNote0Create(pGameInstance->pruntime, \
									& pGameInstance->world, &pGameInstance->pianoModel, pGameInstance->piano_keyboard, \
									x, 0, 10, 100, note, PX_COLOR(192, 255, 255, 255)\
								);
								PX_WorldAddObject(&pGameInstance->world, pNewNote);


								if (note.opcode == PX_MIDI_OPCODE_KEYDOWN)
								{
									px_bool rightHandle = PX_TRUE;
									if (index < 34)
									{
										pGameInstance->last_leftHandle = index;
										rightHandle = PX_FALSE;
									}
									else if (index >= 45)
									{
										pGameInstance->last_rightHandle = index;
									}
									else
									{
										if (PX_ABS(pGameInstance->last_leftHandle - index) > PX_ABS(pGameInstance->last_rightHandle - index))
										{
											pGameInstance->last_rightHandle = index;
										}
										else
										{
											pGameInstance->last_leftHandle = index;
											rightHandle = PX_FALSE;
										}
									}
									if (rightHandle)
									{
										px_dword delay = (px_dword)(pGameInstance->piano_keyboard->y * 10);
										PX_Object_ClickBarTrigger(pGameInstance->clickbar, delay);
									}

								}

							}
						}
					}
					else if (PX_MidiGetNextType(&pGameInstance->Midi, itrack) == PX_MIDI_NEXT_TYPE_TICK)
					{
						tick = PX_MidiReadTick(&pGameInstance->Midi, itrack);
						if (tick == 0)
						{
							continue;
						}
						if (tick == -1)
						{
							break;
						}
						elapsedms = tick * pGameInstance->Midi.Qn * 1.0f / pGameInstance->Midi.TPQN;
						pGameInstance->GameTrackElapsed[itrack] += elapsedms;
						break;
					}
					else if (PX_MidiGetNextType(&pGameInstance->Midi, itrack) == PX_MIDI_NEXT_TYPE_END)
					{
						break;
					}
				}


			}

		}
		PX_WorldUpdate(&pGameInstance->world, PX_GAMEINSTANCE_ELAPSED_ATOM);
	}
}
px_void PX_GameInstanceUpdateConfig(PX_GameInstance* pGameInstance, px_dword elapsed)
{
	PX_WorldUpdate(&pGameInstance->world, elapsed);
}
px_void PX_GameInstanceUpdateLoading(PX_GameInstance* pGameInstance, px_dword elapsed)
{
	PX_WorldUpdate(&pGameInstance->world, elapsed);
}
px_void PX_GameInstanceUpdateCounterDown(PX_GameInstance* pGameInstance, px_dword elapsed)
{
	PX_WorldUpdate(&pGameInstance->world, elapsed);
}
px_void PX_GameInstanceUpdate(PX_GameInstance* pGameInstance, px_dword elapsed)
{
	switch (pGameInstance->state)
	{
	case PX_GAMEINSTANCE_STATE_CONFIG:
		PX_GameInstanceUpdateConfig(pGameInstance, elapsed);
		break;
	case PX_GAMEINSTANCE_STATE_LOADING:
		PX_GameInstanceUpdateLoading(pGameInstance,elapsed);
		break;
	case PX_GAMEINSTANCE_STATE_COUNTERDOWN:
		PX_GameInstanceUpdateCounterDown(pGameInstance, elapsed);
		break;
	case PX_GAMEINSTANCE_STATE_GAMEING:
		PX_GameInstanceUpdateGaming(pGameInstance, elapsed);
		break;
	default:
		break;
	}
}

px_void PX_GameInstanceRender(px_surface* psurface, PX_GameInstance* pGameInstance, px_dword elapsed)
{
	PX_WorldRender(psurface, &pGameInstance->world, elapsed);

}

px_void PX_GameInstancePostEvent(PX_GameInstance* pGameInstance, PX_Object_Event e)
{
	switch (pGameInstance->state)
	{
	case PX_GAMEINSTANCE_STATE_CONFIG:
		PX_ObjectPostEvent(pGameInstance->Config, e);
		break;
	case PX_GAMEINSTANCE_STATE_LOADING:

		break;
	case PX_GAMEINSTANCE_STATE_COUNTERDOWN:

		break;
	case PX_GAMEINSTANCE_STATE_GAMEING:
		PX_ObjectPostEvent(pGameInstance->clickbar, e);
		break;
	default:
		break;
	}

	
}

px_void PX_GameInstanceFree(PX_GameInstance* pGameInstance)
{
}

