#include "PX_Object_GameTuner.h"
volatile px_bool lock = PX_FALSE;

px_void PX_Object_GameTunerOnPianoKey(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object_GameTuner* pDesc = (PX_Object_GameTuner*)ptr;
	px_int index = PX_Object_Event_GetIndex(e);
	pDesc->currentSelectKey = index;
	PX_PianoTriggerIndex(&pDesc->Piano, index, 1);
	PX_Object_PianoTuneSetParameters(pDesc->piano_tune, &pDesc->Piano, index);
	PX_Object_PianoClearKeyState(pDesc->piano_keyboard);
	PX_Object_PianoSetKeyFocus(pDesc->piano_keyboard, index);
}



px_void PX_Object_GameTunerOnPianoReset(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object_GameTuner* pDesc = (PX_Object_GameTuner*)ptr;
	if (pDesc->currentSelectKey != -1)
	{
		PX_PianoKeyFree(&pDesc->Piano.keys[pDesc->currentSelectKey]);
		PX_PianoKeyInitialize(&pDesc->pruntime->mp_game, &pDesc->Piano.keys[pDesc->currentSelectKey], &pDesc->keyparam[pDesc->currentSelectKey]);
		PX_Object_PianoTuneSetParameters(pDesc->piano_tune, &pDesc->Piano, pDesc->currentSelectKey);
	}

}

px_void PX_Object_GameTunerOnPianoPlay(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_ObjectExecuteEvent((PX_Object *)ptr, PX_OBJECT_BUILD_EVENT(PX_OBJECT_GAMETUNE_EVENT_DONE));
}

px_void PX_Object_GameTunerOnPianoSave(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	px_int i;
	PX_Object_GameTuner* pDesc = (PX_Object_GameTuner*)ptr;
	px_memory configData;
	PX_MemoryInitialize(&pDesc->pruntime->mp_game, &configData);
	for (i = 0; i < 88; i++)
	{
		PX_MemoryCat(&configData, &pDesc->Piano.keys[i].param, sizeof(PX_PianoKey_Parameters));
	}
	PX_MemoryCat(&configData, &pDesc->Piano.soundboard.param, sizeof(PX_PianoSoundboard_Parameters));
	if(PX_SaveDataToFile(configData.buffer, configData.usedsize, "assets/model.piano"))
		PX_Object_MessageBoxAlertOk(pDesc->messagebox, "Save piano data to current path \"assets/model.piano\"", 0, 0);
	else
		PX_Object_MessageBoxAlertOk(pDesc->messagebox, "Failed to Save piano data", 0, 0);
}

px_bool PX_Object_GameTunerLoadPianoData(PX_Object_GameTuner* pDesc)
{
	PX_IO_Data io;
	io = PX_LoadFileToIOData("assets/config.piano");
	if (io.size != sizeof(PX_PianoKey_Parameters) * 88 + sizeof(PX_PianoSoundboard_Parameters))
	{
		return PX_FALSE;
	}

	if (!lock)
	{
		PX_PianoKey_Parameters* pkeyparam;
		PX_PianoSoundboard_Parameters* psoundboardparam;
		lock = PX_TRUE;
		pkeyparam = (PX_PianoKey_Parameters*)io.buffer;
		psoundboardparam = (PX_PianoSoundboard_Parameters*)(io.buffer + sizeof(PX_PianoKey_Parameters) * 88);
		PX_PianoFree(&pDesc->Piano);
		PX_PianoInitializeEx(&pDesc->pruntime->mp_game, &pDesc->Piano, pkeyparam, psoundboardparam);
		lock = PX_FALSE;
	}

	PX_FreeIOData(&io);
	return PX_TRUE;
}

px_void PX_Object_GameTunerOnPianoLoad(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object_GameTuner* pDesc = (PX_Object_GameTuner*)ptr;
	if (!PX_Object_GameTunerLoadPianoData(pDesc))
	{
		PX_Object_MessageBoxAlertOk(pDesc->messagebox, "Could not found(or invalid) \"assets/model.piano\"", 0, 0);
	}
	else
	{
		PX_Object_MessageBoxAlertOk(pDesc->messagebox, "Load piano configurations succeeded", 0, 0);
		if(pDesc->currentSelectKey != -1)
		{
			PX_Object_PianoTuneSetParameters(pDesc->piano_tune, &pDesc->Piano, pDesc->currentSelectKey);
		}
	}


}
px_void PX_PianoRead(px_void* userptr, px_byte* pBuffer, px_int readSize)
{
	if (!readSize || readSize > 2048 * 2 * 2)
	{
		PX_memset(pBuffer, 0, readSize);
	}
	else
	{
		px_int oft = 0;
		PX_Object_GameTuner* pDesc = (PX_Object_GameTuner*)userptr;
		px_int count = readSize / 4;//2bytes 2channels
		px_float out44100f[2048];
		px_int i;
		px_short* PCM = (px_short*)pBuffer;
		if (!lock)
		{
			lock = PX_TRUE;
			PX_PianoGo(&pDesc->Piano, out44100f, count);
			lock = PX_FALSE;
		}

		for (i = 0; i < count; i++)
		{
			PCM[oft++] = (px_short)(out44100f[i] * 32768 * pDesc->volume);
			PCM[oft++] = (px_short)(out44100f[i] * 32768 * pDesc->volume);
		}
	}
}

px_void PX_Object_GameTunerOnPianoTuneChanged(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object_GameTuner* pDesc = (PX_Object_GameTuner*)ptr;
	PX_Object_PianoTune* pTune = PX_ObjectGetDesc(PX_Object_PianoTune, pDesc->piano_tune);
	PX_PianoKey* pKey;
	if (pDesc->currentSelectKey == -1)
	{
		return;
	}
	pKey = &pDesc->Piano.keys[pDesc->currentSelectKey];
	if (!PX_memequ(&pTune->key_param, &pKey->param, sizeof(PX_PianoKey_Parameters)))
	{
		if (!lock)
		{
			lock = PX_TRUE;
			PX_PianoKeyFree(pKey);
			if (!PX_PianoKeyInitialize(&pDesc->pruntime->mp_game, pKey, &pTune->key_param))
				PX_ERROR("out of memory");
			lock = PX_FALSE;
		}
	}

	if (!PX_memequ(&pTune->soundboard_param, &pDesc->Piano.soundboard, sizeof(PX_PianoSoundboard_Parameters)))
	{
		if (!lock)
		{
			lock = PX_TRUE;
			PX_PianoSoundBoardFree(&pDesc->Piano.soundboard);
			if (!PX_PianoSoundBoardInitialize(&pDesc->pruntime->mp_game, &pDesc->Piano.soundboard, &pTune->soundboard_param))
				PX_ERROR("out of memory");
			lock = PX_FALSE;
		}
	}
}

px_void PX_Object_GameTunerFree(PX_Object* pObject)
{
	PX_Object_GameTuner* pDesc = PX_ObjectGetDesc(PX_Object_GameTuner, pObject);
	PX_PianoFree(&pDesc->Piano);
	PX_SoundPlaySetUserRead(&pDesc->pruntime->soundplay, 0, 0);
}

px_void PX_Object_GameTunerVolumeChanged(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	px_char content[32] = {0};
	PX_Object_GameTuner* ptuner = PX_ObjectGetDesc(PX_Object_GameTuner, (PX_Object *)ptr);
	ptuner->volume = (px_float)PX_Object_SliderBarGetValue(pObject)/100;
	PX_sprintf1(content, sizeof(content), "Volume:%1.2", PX_STRINGFORMAT_FLOAT(ptuner->volume));
	PX_Object_LabelSetText(ptuner->label_volume, content);
}

px_void PX_Object_GameTunerSpeedChanged(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	px_char content[32] = { 0 };
	PX_Object_GameTuner* ptuner = PX_ObjectGetDesc(PX_Object_GameTuner, (PX_Object*)ptr);
	ptuner->speed = (px_float)PX_Object_SliderBarGetValue(pObject) *1.0f;
	PX_sprintf1(content, sizeof(content), "Speed:%1", PX_STRINGFORMAT_INT((px_int)ptuner->speed));
	PX_Object_LabelSetText(ptuner->label_speed, content);
}
PX_Object* PX_Object_GameTunerCreate(PX_Runtime* pruntime, PX_World* pWorld, PX_PianoKey_Parameters* keyparam, PX_PianoSoundboard_Parameters* soundboardparam,px_int speed)
{

	PX_Object_GameTuner desc,*pDesc;
	PX_Object* pObject;
	PX_memset(&desc, 0, sizeof(desc));
	desc.pruntime = pruntime;
	desc.pWorld = pWorld;
	desc.keyparam = keyparam;
	desc.volume = 1;
	desc.soundboardparam = soundboardparam;
	pObject = PX_ObjectCreateEx(&pruntime->mp_game, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, PX_Object_GameTunerFree, &desc, sizeof(desc));
	pDesc = PX_ObjectGetDesc(PX_Object_GameTuner, pObject);

	PX_PianoInitializeEx(&pDesc->pruntime->mp_game, &pDesc->Piano,keyparam,soundboardparam);
	PX_SoundPlaySetUserRead(&pDesc->pruntime->soundplay, PX_PianoRead, pDesc);

	pDesc->root = PX_Object_WidgetCreate(&pDesc->pruntime->mp_game, pObject, 0, 0, pruntime->surface_width*8/9, pruntime->surface_height * 8 / 9, "Game Piano Tuner", 0);
	PX_Object_WidgetShowHideCloseButton(pDesc->root, PX_FALSE);
	pDesc->root->x = (pruntime->surface_width - pDesc->root->Width) / 2;
	pDesc->root->y = (pruntime->surface_height - pDesc->root->Height) / 2;

	pDesc->button_play = PX_Object_PushButtonCreate(&pDesc->pruntime->mp_ui, pDesc->root, 5, 5, 128, 32, "Go Go Go!!!", PX_NULL);
	PX_Object_PushButtonSetTextColor(pDesc->button_play, PX_COLOR_GREEN);

	pDesc->label_volume = PX_Object_LabelCreate(&pDesc->pruntime->mp_ui, pDesc->root, 152, 2, 100, 18, "Volume:1.00", PX_NULL, PX_OBJECT_UI_DEFAULT_FONTCOLOR);
	pDesc->sliderbar_volume = PX_Object_SliderBarCreate(&pDesc->pruntime->mp_ui, pDesc->root, 260, 5, 192, 12,PX_OBJECT_SLIDERBAR_TYPE_HORIZONTAL,PX_OBJECT_SLIDERBAR_STYLE_BOX);
	PX_Object_SliderBarSetRange(pDesc->sliderbar_volume, 100, 1600);
	PX_Object_SliderBarSetValue(pDesc->sliderbar_volume, 100);
	PX_ObjectRegisterEvent(pDesc->sliderbar_volume, PX_OBJECT_EVENT_VALUECHANGED, PX_Object_GameTunerVolumeChanged, pObject);

	pDesc->label_speed = PX_Object_LabelCreate(&pDesc->pruntime->mp_ui, pDesc->root, 152, 22, 100, 18, "Speed:0", PX_NULL, PX_OBJECT_UI_DEFAULT_FONTCOLOR);
	pDesc->sliderbar_speed = PX_Object_SliderBarCreate(&pDesc->pruntime->mp_ui, pDesc->root, 260, 25, 192, 12, PX_OBJECT_SLIDERBAR_TYPE_HORIZONTAL, PX_OBJECT_SLIDERBAR_STYLE_BOX);
	PX_Object_SliderBarSetRange(pDesc->sliderbar_speed, 0, 1000);
	PX_Object_SliderBarSetValue(pDesc->sliderbar_speed, speed);
	PX_ObjectRegisterEvent(pDesc->sliderbar_speed, PX_OBJECT_EVENT_VALUECHANGED, PX_Object_GameTunerSpeedChanged, pObject);
	PX_ObjectExecuteEvent(pDesc->sliderbar_speed, PX_OBJECT_BUILD_EVENT(PX_OBJECT_EVENT_VALUECHANGED));

	pDesc->button_reset = PX_Object_PushButtonCreate(&pDesc->pruntime->mp_ui, pDesc->root, PX_Object_WidgetGetRenderTargetWidth(pDesc->root) -100, 5, 96, 32, "Reset Key", PX_NULL);
	pDesc->button_save = PX_Object_PushButtonCreate(&pDesc->pruntime->mp_ui, pDesc->root, PX_Object_WidgetGetRenderTargetWidth(pDesc->root) - 200, 5, 96, 32, "Save Model", PX_NULL);
	pDesc->button_load = PX_Object_PushButtonCreate(&pDesc->pruntime->mp_ui, pDesc->root, PX_Object_WidgetGetRenderTargetWidth(pDesc->root) - 300, 5, 96, 32, "Load Model", PX_NULL);
	pDesc->piano_keyboard = PX_Object_PianoCreate(&pDesc->pruntime->mp_ui, pDesc->root, 0, PX_Object_WidgetGetRenderTargetHeight(pDesc->root)-64.f, PX_Object_WidgetGetRenderTargetWidth(pDesc->root)*1.f, 64);

	pDesc->piano_tune = PX_Object_PianoTuneCreate(&pDesc->pruntime->mp_ui, pDesc->root, 0, 24, PX_Object_WidgetGetRenderTargetWidth(pDesc->root)*1.0f, PX_Object_WidgetGetRenderTargetHeight(pDesc->root)-128.f);
	pDesc->messagebox = PX_Object_MessageBoxCreate(&pDesc->pruntime->mp_ui, pDesc->root, PX_NULL);
	pDesc->currentSelectKey = -1;

	PX_ObjectRegisterEvent(pDesc->button_play, PX_OBJECT_EVENT_EXECUTE, PX_Object_GameTunerOnPianoPlay, pObject);
	PX_ObjectRegisterEvent(pDesc->piano_keyboard, PX_OBJECT_EVENT_VALUECHANGED, PX_Object_GameTunerOnPianoKey, pDesc);
	PX_ObjectRegisterEvent(pDesc->button_save, PX_OBJECT_EVENT_EXECUTE, PX_Object_GameTunerOnPianoSave, pDesc);
	PX_ObjectRegisterEvent(pDesc->button_load, PX_OBJECT_EVENT_EXECUTE, PX_Object_GameTunerOnPianoLoad, pDesc);
	PX_ObjectRegisterEvent(pDesc->piano_tune, PX_OBJECT_EVENT_VALUECHANGED, PX_Object_GameTunerOnPianoTuneChanged, pDesc);

	PX_Object_GameTunerLoadPianoData(pDesc);
	return pObject;
}

